from asyncio.coroutines import iscoroutinefunction
from functools import wraps
from importlib import import_module
from typing import Any, Callable, Coroutine, TypeVar, overload


def import_obj(path: str, obj_name: str = None) -> Any:
    """Import an object from a string path."""
    module_path, _, obj_path = path.rpartition(".")
    try:
        module = import_module(module_path)
        return getattr(module, obj_path)
    except (ImportError, AttributeError, ValueError) as exc:
        raise ImportError(f"Could not import {path!r}") from exc


TRes = TypeVar("TRes")
TCoroFn = TypeVar("TCoroFn", bound=Callable[..., Coroutine])


@overload
def to_coroutinefn(fn: TCoroFn) -> TCoroFn:
    ...


@overload
def to_coroutinefn(fn: Callable[..., TRes]) -> Callable[..., Coroutine[Any, Any, TRes]]:
    ...


def to_coroutinefn(fn):
    """Convert a function to a coroutine function."""
    if iscoroutinefunction(fn):
        return fn

    @wraps(fn)
    async def corofn(*args, **kwargs):
        return fn(*args, **kwargs)

    return corofn



from __future__ import annotations

from numbers import Number
from typing import Any, Awaitable, Callable, Coroutine, Dict, Literal, Optional, Tuple, TypedDict

TBackendType = Literal["memory", "redis", "amqp"]

TTaskParams = TypedDict(
    "TTaskParams",
    {
        "bind": Optional[bool],
        "delay": Optional[Number],
        "timeout": Optional[Number],
    },
)

TRunArgs = Tuple[str, Tuple, Dict[str, Any], TTaskParams]

TWorkerParams = TypedDict(
    "TWorkerParams",
    {
        "max_tasks": Optional[int],
        "task_defaults": Optional[TTaskParams],
        "on_start": Optional[Callable[[], Awaitable]],
        "on_stop": Optional[Callable[[], Awaitable]],
        "on_error": Optional[Callable[[BaseException], Coroutine]],
        "show_banner": Optional[bool],
        # "timeout": Number,
        # "task_retry": int,
        # "task_retry_delay": Number,
    },
)

TManagerParams = TypedDict(
    "TManagerParams",
    {
        "backend": TBackendType,
        "backend_params": Dict,
        "log_config": Optional[Dict],
        "log_level": str,
        "worker_params": TWorkerParams,
    },
)
